@@ -22,11 +22,11 @@
 #' 
 #' Saves (serializes) an object to disk.  
 #' @usage qsave(x, file, 
-#' preset = "balanced", algorithm = "lz4", compress_level = 1L, 
+#' preset = "high", algorithm = "zstd", compress_level = 4L, 
 #' shuffle_control = 15L, check_hash=TRUE, nthreads = 1)
 #' @param x the object to serialize.
 #' @param file the file name/path.
-#' @param preset One of "fast", "balanced" (default), "high", "archive", "uncompressed" or "custom".  See details.  
+#' @param preset One of "fast", "high" (default), "high", "archive", "uncompressed" or "custom".  See details.  
 #' @param algorithm Compression algorithm used: "lz4", "zstd", "lz4hc", "zstd_stream" or "uncompressed".
 #' @param compress_level The compression level used (Default 1).  For lz4, this number must be > 1 (higher is less compressed).  For zstd, a number between -50 to 22 (higher is more compressed).  
 #' @param shuffle_control An integer setting the use of byte shuffle compression.  A value between 0 and 15 (Default 3).  See details.  
@@ -79,8 +79,8 @@
 #' w2 <- qread(myfile)
 #' identical(w, w2) # returns true
 #' @export
-qsave <- function(x, file, preset="balanced", algorithm="lz4", compress_level=1L, shuffle_control=15L, check_hash = TRUE, nthreads=1) {
-  invisible(c_qsave(x,normalizePath(file, mustWork=FALSE), preset, algorithm, compress_level, shuffle_control, check_hash, nthreads))
+qsave <- function(x, file, preset="high", algorithm="zstd", compress_level=4L, shuffle_control=15L, check_hash = TRUE, nthreads=1) {
+  invisible(c_qsave(x, file, preset, algorithm, compress_level, shuffle_control, check_hash, nthreads))
 }
 
 #' qread
@@ -120,18 +120,18 @@ qsave <- function(x, file, preset="balanced", algorithm="lz4", compress_level=1L
 #' identical(w, w2) # returns true
 #' @export
 qread <- function(file, use_alt_rep=FALSE, strict=FALSE, nthreads=1) {
-  c_qread(normalizePath(file, mustWork=FALSE), use_alt_rep, strict, nthreads)
+  c_qread(file, use_alt_rep, strict, nthreads)
 }
 
 #' qsave_fd
 #' 
 #' Saves an object to a file descriptor
 #' @usage qsave_fd(x, fd, 
-#' preset = "balanced", algorithm = "lz4", compress_level = 1L, 
+#' preset = "high", algorithm = "zstd", compress_level = 4L, 
 #' shuffle_control = 15L, check_hash=TRUE)
 #' @param x the object to serialize.
 #' @param fd A file descriptor
-#' @param preset One of "fast", "balanced" (default), "high", "archive", "uncompressed" or "custom".  See details.  
+#' @param preset One of "fast", "balanced" , "high" (default), "archive", "uncompressed" or "custom".  See details.  
 #' @param algorithm Compression algorithm used: "lz4", "zstd", "lz4hc", "zstd_stream" or "uncompressed".
 #' @param compress_level The compression level used (Default 1).  For lz4, this number must be > 1 (higher is less compressed).  For zstd, a number between -50 to 22 (higher is more compressed).  
 #' @param shuffle_control An integer setting the use of byte shuffle compression.  A value between 0 and 15 (Default 3).  See details.  
@@ -141,7 +141,7 @@ qread <- function(file, use_alt_rep=FALSE, strict=FALSE, nthreads=1) {
 #' This function serializes and compresses an R object to a stream using a file descriptor
 #' If your data is important, make sure you know what happens on the other side of the pipe.  See examples for usage.   
 #' @export
-qsave_fd <- function(x, fd, preset = "balanced",  algorithm="lz4", compress_level=1L, shuffle_control=15L, check_hash = TRUE) {
+qsave_fd <- function(x, fd, preset = "high",  algorithm="zstd", compress_level=4L, shuffle_control=15L, check_hash = TRUE) {
   invisible(c_qsave_fd(x, fd, preset, algorithm, compress_level, shuffle_control, check_hash))
 }
 
@@ -165,11 +165,11 @@ qread_fd <- function(fd, use_alt_rep=FALSE, strict=FALSE) {
 #' 
 #' Saves an object to a windows handle
 #' @usage qsave_handle(x, handle, 
-#' preset = "balanced", algorithm = "lz4", compress_level = 1L, 
+#' preset = "high", algorithm = "zstd", compress_level = 4L, 
 #' shuffle_control = 15L, check_hash=TRUE)
 #' @param x the object to serialize.
 #' @param handle A windows handle external pointer
-#' @param preset One of "fast", "balanced" (default), "high", "archive", "uncompressed" or "custom".  See details.  
+#' @param preset One of "fast", "balanced" , "high" (default), "archive", "uncompressed" or "custom".  See details.  
 #' @param algorithm Compression algorithm used: "lz4", "zstd", "lz4hc", "zstd_stream" or "uncompressed".
 #' @param compress_level The compression level used (Default 1).  For lz4, this number must be > 1 (higher is less compressed).  For zstd, a number between -50 to 22 (higher is more compressed).  
 #' @param shuffle_control An integer setting the use of byte shuffle compression.  A value between 0 and 15 (Default 3).  See details.  
@@ -179,7 +179,7 @@ qread_fd <- function(fd, use_alt_rep=FALSE, strict=FALSE) {
 #' This function serializes and compresses an R object to a stream using a file descriptor
 #' If your data is important, make sure you know what happens on the other side of the pipe.  See examples for usage.   
 #' @export
-qsave_handle <- function(x, handle, preset = "balanced",  algorithm="lz4", compress_level=1L, shuffle_control=15L, check_hash = TRUE) {
+qsave_handle <- function(x, handle, preset = "high",  algorithm="zstd", compress_level=4L, shuffle_control=15L, check_hash = TRUE) {
   invisible(c_qsave_handle(x, handle, preset, algorithm, compress_level, shuffle_control, check_hash))
 }
 
@@ -203,11 +203,11 @@ qread_handle <- function(handle, use_alt_rep=FALSE, strict=FALSE) {
 #' qserialize
 #' 
 #' Saves an object to a raw vector 
-#' @usage qserialize(x, preset = "balanced", 
-#' algorithm = "lz4", compress_level = 1L, 
+#' @usage qserialize(x, preset = "high", 
+#' algorithm = "zstd", compress_level = 4L, 
 #' shuffle_control = 15L, check_hash=TRUE)
 #' @param x the object to serialize.
-#' @param preset One of "fast", "balanced" (default), "high", "archive", "uncompressed" or "custom".  See details.  
+#' @param preset One of "fast", "balanced", "high" (default), "archive", "uncompressed" or "custom".  See details.  
 #' @param algorithm Compression algorithm used: "lz4", "zstd", "lz4hc", "zstd_stream" or "uncompressed".
 #' @param compress_level The compression level used (Default 1).  For lz4, this number must be > 1 (higher is less compressed).  For zstd, a number between -50 to 22 (higher is more compressed).  
 #' @param shuffle_control An integer setting the use of byte shuffle compression.  A value between 0 and 15 (Default 3).  See details.  
@@ -216,7 +216,7 @@ qread_handle <- function(handle, use_alt_rep=FALSE, strict=FALSE) {
 #' This function serializes and compresses an R object to a raw vctor
 #' If your data is important, make sure you know what happens on the other side of the pipe.  See examples for usage.   
 #' @export
-qserialize <- function(x, preset = "balanced",  algorithm="lz4", compress_level=1L, shuffle_control=15L, check_hash = TRUE) {
+qserialize <- function(x, preset = "high",  algorithm="zstd", compress_level=4L, shuffle_control=15L, check_hash = TRUE) {
   c_qserialize(x, preset, algorithm, compress_level, shuffle_control, check_hash)
 }
 
