@@ -19,8 +19,9 @@
  */
 
 #include "qs_common.h"
+#include "qs_deserialize_common.h"
 
-static constexpr uint64_t RESERVE_SIZE = 4; // 2^17
+static constexpr uint64_t RESERVE_SIZE = 4;
 
 template <class stream_reader>
 struct ZSTD_streamRead {
@@ -314,6 +315,7 @@ struct Data_Context_Stream {
   QsMetadata qm;
   DestreamClass & dsc;
   bool use_alt_rep_bool;
+  std::unordered_map<uint32_t, SEXP> object_ref_hash;
   std::vector<uint8_t> shuffleblock = std::vector<uint8_t>(256);
   uint64_t & data_offset; // dsc.blockoffset
   uint64_t & block_size; // dsc.blocksize
@@ -329,7 +331,7 @@ struct Data_Context_Stream {
   void getBlockData(char* outp, uint64_t data_size) {
     dsc.copyData(outp, data_size);
   }
-  void readHeader(SEXPTYPE & object_type, uint64_t & r_array_len) {
+  void readHeader(qstype & object_type, uint64_t & r_array_len) {
     if(data_offset + BLOCKRESERVE >= block_size) getBlock();
     readHeader_common(object_type, r_array_len, data_offset, data_ptr);
   }
@@ -347,156 +349,5 @@ struct Data_Context_Stream {
       getBlockData(outp, data_size);
     }
   }
-  SEXP processBlock() {
-    SEXPTYPE obj_type;
-    uint64_t r_array_len;
-    readHeader(obj_type, r_array_len);
-    // std::cout << r_array_len << " " << obj_type << "\n";
-    uint64_t number_of_attributes;
-    if(obj_type == ANYSXP) {
-      number_of_attributes = r_array_len;
-      readHeader(obj_type, r_array_len);
-      // std::cout << r_array_len << " " << obj_type << "\n";
-    } else {
-      number_of_attributes = 0;
-    }
-    SEXP obj;
-    Protect_Tracker pt = Protect_Tracker();
-    switch(obj_type) {
-    case VECSXP: 
-      obj = PROTECT(Rf_allocVector(VECSXP, r_array_len)); pt++;
-      for(uint64_t i=0; i<r_array_len; i++) {
-        SET_VECTOR_ELT(obj, i, processBlock());
-      }
-      break;
-    case REALSXP:
-      obj = PROTECT(Rf_allocVector(REALSXP, r_array_len)); pt++;
-      if(qm.real_shuffle) {
-        getShuffleBlockData(reinterpret_cast<char*>(REAL(obj)), r_array_len*8, 8);
-      } else {
-        getBlockData(reinterpret_cast<char*>(REAL(obj)), r_array_len*8);
-      }
-      break;
-    case INTSXP:
-      obj = PROTECT(Rf_allocVector(INTSXP, r_array_len)); pt++;
-      if(qm.int_shuffle) {
-        getShuffleBlockData(reinterpret_cast<char*>(INTEGER(obj)), r_array_len*4, 4);
-      } else {
-        getBlockData(reinterpret_cast<char*>(INTEGER(obj)), r_array_len*4);
-      }
-      break;
-    case LGLSXP:
-      obj = PROTECT(Rf_allocVector(LGLSXP, r_array_len)); pt++;
-      if(qm.lgl_shuffle) {
-        getShuffleBlockData(reinterpret_cast<char*>(LOGICAL(obj)), r_array_len*4, 4);
-      } else {
-        getBlockData(reinterpret_cast<char*>(LOGICAL(obj)), r_array_len*4);
-      }
-      break;
-    case CPLXSXP:
-      obj = PROTECT(Rf_allocVector(CPLXSXP, r_array_len)); pt++;
-      if(qm.cplx_shuffle) {
-        getShuffleBlockData(reinterpret_cast<char*>(COMPLEX(obj)), r_array_len*16, 8);
-      } else {
-        getBlockData(reinterpret_cast<char*>(COMPLEX(obj)), r_array_len*16);
-      }
-      break;
-    case RAWSXP:
-      obj = PROTECT(Rf_allocVector(RAWSXP, r_array_len)); pt++;
-      if(r_array_len > 0) getBlockData(reinterpret_cast<char*>(RAW(obj)), r_array_len);
-      break;
-    case STRSXP:
-#ifdef ALTREP_SUPPORTED
-      if(use_alt_rep_bool) {
-        auto ret = new stdvec_data(r_array_len);
-        for(uint64_t i=0; i < r_array_len; i++) {
-          uint32_t r_string_len;
-          cetype_t string_encoding = CE_NATIVE;
-          readStringHeader(r_string_len, string_encoding);
-          // std::cout << "string " << r_string_len << " " << string_encoding << "\n";
-          if(r_string_len == NA_STRING_LENGTH) {
-            ret->encodings[i] = 5;
-          } else if(r_string_len == 0) {
-            ret->encodings[i] = 1;
-            ret->strings[i] = "";
-          } else {
-            switch(string_encoding) {
-            case CE_NATIVE:
-              ret->encodings[i] = 1;
-              break;
-            case CE_UTF8:
-              ret->encodings[i] = 2;
-              break;
-            case CE_LATIN1:
-              ret->encodings[i] = 3;
-              break;
-            case CE_BYTES:
-              ret->encodings[i] = 4;
-              break;
-            default:
-              ret->encodings[i] = 5;
-            break;
-            }
-            ret->strings[i].resize(r_string_len);
-            getBlockData(&(ret->strings[i])[0], r_string_len);
-          }
-        }
-        obj = PROTECT(stdvec_string::Make(ret, true)); pt++;
-      } else {
-#endif
-        obj = PROTECT(Rf_allocVector(STRSXP, r_array_len)); pt++;
-        for(uint64_t i=0; i<r_array_len; i++) {
-          uint32_t r_string_len;
-          cetype_t string_encoding = CE_NATIVE;
-          readStringHeader(r_string_len, string_encoding);
-          // std::cout << "string " << r_string_len << " " << string_encoding << "\n";
-          if(r_string_len == NA_STRING_LENGTH) {
-            SET_STRING_ELT(obj, i, NA_STRING);
-          } else if(r_string_len == 0) {
-            SET_STRING_ELT(obj, i, Rf_mkCharLen("", 0));
-          } else if(r_string_len > 0) {
-            if(r_string_len > temp_string.size()) {
-              temp_string.resize(r_string_len);
-            }
-            getBlockData(&temp_string[0], r_string_len);
-            SET_STRING_ELT(obj, i, Rf_mkCharLenCE(temp_string.data(), r_string_len, string_encoding));
-          }
-        }
-#ifdef ALTREP_SUPPORTED
-      }
-#endif
-      break;
-    case S4SXP:
-    {
-      SEXP obj_data = PROTECT(Rf_allocVector(RAWSXP, r_array_len)); pt++;
-      getBlockData(reinterpret_cast<char*>(RAW(obj_data)), r_array_len);
-      obj = PROTECT(unserializeFromRaw(obj_data)); pt++;
-      
-      return obj;
-    }
-    default: // also NILSXP
-      obj = R_NilValue;
-      return obj;
-    }
-    if(number_of_attributes > 0) {
-      for(uint64_t i=0; i<number_of_attributes; i++) {
-        uint32_t r_string_len;
-        cetype_t string_encoding;
-        readStringHeader(r_string_len, string_encoding);
-        // std::cout << "attribute " << r_string_len << " " << string_encoding << "\n";
-        if(r_string_len > temp_string.size()) {
-          temp_string.resize(r_string_len);
-        }
-        std::string temp_attribute_string = std::string(r_string_len, '\0');
-        getBlockData(&temp_attribute_string[0], r_string_len);
-        // Rf_install may allocate, therefore we need to protect the result of processBlock
-        // Is this really true?  Could be slow with lots of attributes
-        // ref: https://github.com/kalibera/rchk/blob/master/doc/USAGE.md
-        SEXP attrib_obj = PROTECT(processBlock()); pt++;
-        Rf_setAttrib(obj, Rf_install(temp_attribute_string.data()), attrib_obj);
-      }
-    }
-    
-    return obj;
-  }
 };
+;
